@@ -1,10 +1,7 @@
 doubexp <-
 function()
 { 
-   linkfun <- function(mu) { leng <- length(mu)
-                 sapply(1:leng, function(i) {
-                    if (mu[i]<0.5) { log(2*mu[i]) 
-                            } else { abs(log(2*(1-mu[i]))) }} ) }
+   linkfun <- function(mu) { ifelse( (mu<0.5), log(2*mu), abs(log(2*(1-mu))) ) }  
    linkinv <- function(eta) { (1+sign(eta))/2 - sign(eta)*exp(-abs(eta))/2 }
    mu.eta <- function(eta) { exp(-abs(eta))/2 }
    valideta <- function(eta) TRUE
